#!/usr/bin/env python

# This script takes input in the format described below and uses it to build
# `\pdfoutline' primitives (pdfTeX primitive for PDF bookmarks), writing them to
# stdout.

# Command line arguments:
#   * no arguments means read from stdin;
#   * first argument is interpreted as input file name;
#   * specifying `-' as the first argument means read from stdin.

# Format of input.  All lines are in groups of three:
#   * first line of a group must be a decimal integer; its absolute value
#     indicates level of the current bookmark entry, and its sign indicates
#     whether bookmark's sub-entries should be expanded (positive) or collapsed
#     (negative).  Note that entries don't have to start on level 1, and
#     levels don't have to grow by increments of 1 (for example, you can give
#     level 1 for one bookmark, and 3 for the next);
#   * second line of a group must contain first part of the parameters for
#     `\pdfoutline' primitive (everything up to bookmark's text string),
#     without `\pdfoutline' word and without `count N' parameter (the latter
#     will be calculated by this script on the basis of outline levels);
#   * third line of a group must contain bookmark's text string for
#     `\pdfoutline' primitive (if needed, it has to be translated into Unicode
#     beforehand, see `tounicode.py' script).

import sys

# Global vars we use and their initial values
lineno     = 0
infilename = "-"
infile     = sys.stdin
outfile    = sys.stdout

# These four global variables are used for communication with readentry()
Level            = 0
ExpandSubentries = 0
Preamble         = ""
Text             = ""

TeXescapechar = '\\' # escape character for TeX commands


# Reads in a line from infile and strips terminating newline, if any.  If string
# ErrorMsg is given, fails when no more lines are available, displays error
# message on stderr, inserting ErrorMsg as an explanation, and bails out with
# exit code 1.  If ErrorMsg is not given, just returns None instead of string to
# indicate failed read
def readinline(ErrorMsg = None):
    global infilename, infile, outfile, lineno

    tmp = infile.readline()
    if tmp == "": # this is EOF
        if ErrorMsg is None:
            return None
        else:
            outfile.flush()
            sys.stderr.write( \
              "%s: file `%s', unexpected end of file (%s)\n" % \
              (sys.argv[0], infilename, ErrorMsg) )
            sys.exit(1)

    lineno += 1

    # Eat up terminating newline, if any
    if tmp.endswith('\n'):
        tmp = tmp[:-1]

    return tmp


# Reads in three lines from input file, processes them, and saves results in
# Level, ExpandSubentries, Preamble and Text variables.  If no more entries are
# available from the input file, sets Level to None
def readentry():
    global lineno, infilename, outfile
    global Level, ExpandSubentries, Preamble, Text
    global TeXescapechar

    # Try and read level number
    tmp = readinline()
    if tmp is None: # this is EOF
        Level = None
        return

    # This is not EOF, go on
    try:
        Level = int(tmp)
    except ValueError:
        outfile.flush()
        sys.stderr.write( \
          "%s: file `%s', error parsing line %d (integer expected)\n" % \
          (sys.argv[0], infilename, lineno) )
        sys.exit(1)

    if Level == 0:
        outfile.flush()
        sys.stderr.write( \
          "%s: file `%s', error parsing line %d (non-zero level number expected)\n" % \
          (sys.argv[0], infilename, lineno) )
        sys.exit(1)
    elif Level > 0:
        ExpandSubentries = 1
    else:
        ExpandSubentries = 0
        Level = -Level

    # Try and read \pdfoutline preamble
    Preamble = readinline("bookmark preamble expected")

    # Read \pdfoutline text
    Text =  readinline("bookmard text expected")

    return


# This function recursively iterates over the input and builds a string which is
# a sequence of \pdfoutline primitives forming a PDF outline tree.  It stops
# iterating after having read an entry with level less or equal to that for
# which it was called.

# Returns the string of \pdfoutline primitives

def buildtree():
    global Level, ExpandSubentries, Preamble, Text

    outstr = ""
    count  = 0
    # Save this bookmark's parameters before reading next entry
    thisLevel            = Level
    thisExpandSubentries = ExpandSubentries
    thisPreamble         = Preamble
    thisText             = Text
    # Read in next bookmark
    readentry()

    # Loop until there are any bookmarks and until they are on an inner level
    while Level is not None and Level > thisLevel:
        count  += 1            # update tree count
        outstr += buildtree()  # add subtree

    if not thisExpandSubentries:
        count = -count
    # Construct and return tree for this entry
    return "%spdfoutline %s count %d {%s}\n%s" % \
           (TeXescapechar, thisPreamble, count, thisText, outstr)


#
# Execution starts here
#

# Open input file
lineno = 0

if len(sys.argv) < 2:
    infilename = "-"
else:
    infilename = sys.argv[1]

if infilename == "-":
    infile = sys.stdin
else:
    try:
        infile = open(infilename, 'r')
    except IOError, (errno, strerror):
        outfile.flush()
        sys.stderr.write( \
          "%s: error opening file `%s' (%s)\n" % \
          (sys.argv[0], infilename, strerror) )
        sys.exit(1)

# Read in first entry
readentry()

# Trigger the recursion
while Level is not None:
    outfile.write(buildtree())

outfile.flush()



# Local variables:
# mode: python
# compile-command: "python ./tounicode.py elsgolts.diff.ur.var.is.bm.in | python ./compilebm.py > elsgolts.diff.ur.var.is.bm"
# End:
