#!/usr/bin/env python

# This script translates bookmark texts from intermediate bookmark file
# (produced by TeX macros) into Unicode.

# Command line arguments are:
#   * first argument (required)--encoding of PDF strings (empty string for a
#     default, usually `ascii', which allows only the first 128 ASCII characters
#     and throws exception if any other encountered);
#   * second argument (optional)--input file name (omit or say `-' to read from
#     stdin).

# Format of input:
#   lines must be in groups of three, and the third (bookmark text) line of each
#   group will be translated into Unicode.

# Format of output:
#   output goes to stdout and is identical to the input except that every third
#   line (bookmark text) is replaced by its Unicode equivalent (see below).

# Translation of bookmark texts into Unicode is done using specified encoding,
# with each character translated into a Unicode character, a sequence of two
# octal numbers in the form `\OOO\OOO', where O is an octal digit.  In the
# output, the first two bytes of the string are the Unicode Byte Order marker
# (\376\377).

import sys

# Global vars we use and their initial values
hexdigits  = "0123456789abcdef"
lineno     = 0
infilename = "-"
infile     = sys.stdin
outfile    = sys.stdout
encoding   = ""

PDFescapechar = "\\string\\\\" # escape character to put in front of each
                               # Unicode char in PDF's Unicode string


# Reads in a line from infile, without stripping terminating newline.  If string
# ErrorMsg is given, fails when no more lines are available, displays error
# message on stderr, inserting ErrorMsg as an explanation, and bails out with
# exit code 1.  If ErrorMsg is not given, just returns None instead of string to
# indicate failed read
def readinline(ErrorMsg = None):
    global infilename, infile, outfile, lineno

    tmp = infile.readline()
    if tmp == "": # this is EOF
        if ErrorMsg is None:
            return None
        else:
            outfile.flush()
            sys.stderr.write( \
              "%s: %s: unexpected end of file (%s)\n" % \
              (sys.argv[0], infilename, ErrorMsg) )
            sys.exit(1)

    lineno += 1

    return tmp


# Translates the string into Unicode writing the result to `outfile'
def writeUnicode(Text):
    global encoding, PDFescapechar, infilename, outfile, lineno

    # Strip terminating newline, if any
    if Text.endswith('\n'):
        Text = Text[:-1]
    # Translate text to Unicode
    try:
        if encoding == "":
            unistr = unicode(Text) # force using default
        else:
            unistr = unicode(Text, encoding)
    except LookupError:
        outfile.flush()
        sys.stderr.write( \
          "%s: %s: unknown encoding `%s'\n" % \
          (sys.argv[0], infilename, encoding) )
        sys.exit(1)
    except ValueError:
        outfile.flush()
        sys.stderr.write( \
          "%s: %s: error translating line %d to Unicode (invalid characters)\n" % \
          (sys.argv[0], infilename, lineno) )
        sys.exit(1)

    outfile.write( "%s376%s377" % (PDFescapechar, PDFescapechar) ) # Unicode Byte Order marker
    # Translate each character into `\OOO\OOO' pair
    for c in unistr:
        outfile.write( "%s%03o%s%03o" % ( PDFescapechar, ord(c)/0x100, \
                                          PDFescapechar, ord(c)%0x100 ) )
    outfile.write('\n')

    return


#
# Execution starts here
#
lineno = 0

# Get input encoding
if len(sys.argv) < 2:
    sys.stderr.write( "%s: error, at least one argument expected (encoding)\n")
    sys.exit(1)

encoding = sys.argv[1].strip()
if encoding == "": # blank line means use default
    outfile.flush()
    sys.stderr.write( \
      "%s: %s: input encoding not specified, using default (%s)\n" % \
      (sys.argv[0], infilename, sys.getdefaultencoding()) )

# Get input file
if len(sys.argv) < 3:
    infilename = "-"
else:
    infilename = sys.argv[2]

if infilename == "-":
    infile = sys.stdin
else:
    try:
        infile = open(infilename, 'r')
    except IOError, (errno, strerror):
        outfile.flush()
        sys.stderr.write( \
          "%s: %s: error opening file (%s)\n" % \
          (sys.argv[0], infilename, strerror) )
        sys.exit(1)

# Read lines by groups of three, writing out first two as is and translating the
# third into Unicode
while 1:
    # Read a group of three lines
    Line = readinline() # read level number
    if Line is None: # this is EOF
        break
    outfile.write(Line)

    Line = readinline("bookmark preamble expected") # read preamble
    outfile.write(Line)

    Line = readinline("bookmark text expected") # read text
    writeUnicode(Line)

outfile.flush()
sys.exit(0)



# Local variables:
# mode: python
# compile-command: "python ./tounicode.py elsgolts.diff.ur.var.is.bm.in"
# End:
